

         /*********************************************************/
         /*                                                       */
         /*                 COPYRIGHT (C) 1988 BY                 */
         /* CAD Group of Dipartimento di Automatica e Informatica */
         /*             Politecnico di Torino, Italy.             */
         /*                  ALL RIGHTS RESERVED.                 */
         /*                                                       */
         /* This software is furnished under a license and may be */
         /* used and copied  only in accordance with the terms of */
         /* such license. Unauthorized copies will be prosecuted. */
         /*                                                       */
         /*********************************************************/





/******************************************************************************

    Filename:         common.h

    Author:           Pier Luca Montessoro

    Creation date:    11 January 1988

    Contents:         common definitions


                          *** DOCUMENTATION ***

      This file contains miscellaneous definitions used by different sections
 of the simulator. In particular, sets of codes used as parameters in function
 calls are defined if they are used as bridge between the sections.

 IMPORTANT:  All the codes are defined and used as 'unsigned char'. This
             ensures the correct parameters passing between functions also
             on 16 bits word-width machines.


 ******************************************************************************/



/* maximum data size (bits) */

#define MAX_DATA_SIZE   16


/* initialization value (unknown status for all the bits) */

#define INITIALIZATION_VALUE  (unsigned long) 0


/* logical values definition. In the whole simulator all the logical values
   are represented vectorially on 32 bits, even if most of all are unused.
   The most significant 16 bits indicates only if the value is defined in
   the boolead (two values) algebra.
   Therefore, the following definitions are more formal than operative. */

/* one */
#define _0_    (((unsigned long) 0) | (((unsigned long ) 1) << MAX_DATA_SIZE))

/* zero */
#define _1_    (((unsigned long) 1) | (((unsigned long ) 1) << MAX_DATA_SIZE))

/* unknown */
#define _X_    ((unsigned long)  0)

/* high impedance */
#define _Z_    ((unsigned long)  1)


/* mask to isolate one-bit value */
#define _MASK (((unsigned long ) 1) | (((unsigned long ) 1) << MAX_DATA_SIZE))


/* macro definition to get the less significant part of the value field
   (the value in 2-values algebra) of the machine descriptors */

#define BOOLEAN_HALF(x)  ((x) & (unsigned long) 0XFFFF)


/* macro definition to get the most significant part of the value field
   (containing the information about X and Z). */

#define XZ_HALF(x)  (((unsigned long) (x)) >> MAX_DATA_SIZE)


/* macro definition to convert an encoded (scalar) value into two adjacent 
   bits */

#define SCALAR_CONVERT(x)  ( (((unsigned long) (x)) & 1) | \
                             ((((unsigned long) (x)) & \
                               (((unsigned long) 1) << MAX_DATA_SIZE)) \
                                                      >> (MAX_DATA_SIZE - 1)) )


/* this macro selects the rise or fall delay. Note that the test is
   performed only on the less significant data bit.
   The the transition delays are:

   _0_ -> _1_ (1...0 -> 1...1)    RISE  (*)
   _0_ -> _X_ (1...0 -> 0...0)    RISE
   _0_ -> _Z_ (1...0 -> 0...1)    RISE  (*)
   _X_ -> _1_ (0...0 -> 1...1)    RISE  (*)
   _X_ -> _Z_ (0...0 -> 0...1)    RISE  (*)
   _Z_ -> _1_ (0...1 -> 1...1)    RISE
              
   _X_ -> _0_ (0...0 -> 1...0)    FALL
   _Z_ -> _0_ (0...1 -> 1...0)    FALL  (*)
   _Z_ -> _X_ (0...1 -> 0...0)    FALL  (*)
   _1_ -> _0_ (1...1 -> 1...0)    FALL  (*)
   _1_ -> _X_ (1...1 -> 0...0)    FALL  (*)
   _1_ -> _Z_ (1...1 -> 0...1)    FALL

   If the less significant bit of the old and new values are different (*),
   the delay value may be decided without testing the other bit. This makes
   very efficient the test when nor X nor Z are present. */

#define IS_RISE_DELAY(old_value,new_value) \
        ( ((old_value & 1) != (new_value & 1)) ? \
           (new_value & 1) \
           : \
           ((old_value & 1) ^ ((old_value >> MAX_DATA_SIZE) & 1)) )


/* maximum strength */

#define MAX_STRENGTH   (unsigned char) 7


/* maximum size */

#define MAX_SIZE       (unsigned char) 7


/* the following set of macros is used to handle the strength information
   contained in the topological descriptor, when static, or in the status
   descriptor when dynamic */

#if LIMITED_MACRO_LENGTH

extern unsigned char  SELECT_STRENGTH();
extern unsigned char  _1_STRENGTH();
extern unsigned char  _0_STRENGTH();
extern unsigned char  DYN_STRENGTH();
extern unsigned char  ST_STRENGTH();
extern unsigned char  STRENGTH();

#endif

#if !LIMITED_MACRO_LENGTH

#define SELECT_STRENGTH(value,bit,_0_strength,_1_strength) \
        ((((value) & (_MASK << bit)) == (_0_ << bit)) ? \
          (unsigned char) (_0_strength) : \
          ((((value) & (_MASK << bit)) == (_1_ << bit)) ? \
            (unsigned char) (_1_strength) : \
            ((((value) & (_MASK << bit)) == (_X_ << bit)) ? \
              ((_0_strength > _1_strength) ? \
               (unsigned char) (_0_strength) : (unsigned char) (_1_strength)) \
              : (unsigned char) 0)))

#define _1_STRENGTH(strength_field) \
        ((unsigned char) ((strength_field) >> 4))

#define _0_STRENGTH(strength_field) \
        ((unsigned char) ((strength_field) & 0xF))

#define DYN_STRENGTH(value) \
        ((unsigned char) ((value) >> 24))

#define ST_STRENGTH(value,bit,strength_field) \
        SELECT_STRENGTH (value, bit, _0_STRENGTH (strength_field), \
                         _1_STRENGTH (strength_field))

#define STRENGTH(value,bit,element_descriptor) \
        (((element_descriptor)->data_type == SWITCH_MODEL) ? \
         DYN_STRENGTH (value) : \
         ST_STRENGTH (value, bit, (element_descriptor)->driving_capabilities))

#endif

#define SET_ST_STRENGTH(strength_field,_0_strength,_1_strength) \
        strength_field = (_0_strength) | ((_1_strength) << 4)

#define SET_DYN_STRENGTH(value_field,strength) \
        value_field = ((value_field) & _MASK) | \
                      ((unsigned long) (strength) << 24)


/* codes for value types */

#define  BOOLEAN   (unsigned char) 0
#define  E_LOGIC   (unsigned char) 1


/* structure definition for value information passing between functions:

   value                is the data field.

   _0_strenght,
   _1_strenght          is the strength information, if present.

   type                 is the code of the value type.

   with_strengt         is a flag to signal that the strength information
                        is present. */

typedef struct Value_Dsc
        {
           unsigned long     value;

           unsigned char     _0_strength;
           unsigned char     _1_strength;
           unsigned char     type;
           unsigned char     with_strength;

        }  VALUE_DSC;


/* time queue elements code definitions (codes 0 thru 4 are reserved for
   internal elements and command line interpreter activation) */

#define EVENT  (unsigned char)  5      /* Event */
#define WSL    (unsigned char)  6      /* WSL activator */
#define CKS    (unsigned char)  7      /* CKS activator */
#define FDROP  (unsigned char)  8      /* Fault Dropping activator */


/* command line interpreters activation modes */

#define SETUP      (unsigned char)  0
#define USER       (unsigned char)  1
#define CONTROL_C  (unsigned char)  2
#define END_WAVE   (unsigned char)  3


/* exit codes */

#define NORMAL                  (unsigned char)  0
#define WRONG_ARGUMENT_NUMBER   (unsigned char)  1
#define FILE_ERROR              (unsigned char)  2
#define SYS_ERROR               (unsigned char)  3
#define BUG                     (unsigned char)  4
#define NOT_YET_IMPLEMENTED     (unsigned char)  5


/* function return codes */

#define RET_SUCCESS             (unsigned char)  0
#define RET_INFORMATIONAL       (unsigned char)  1
#define RET_WARNING             (unsigned char)  2
#define RET_ERROR               (unsigned char)  3


/* statistical items */

#define CPU_TIME                (unsigned char)  0
#define SETUP_CPU_TIME          (unsigned char)  1
#define INSERTED_EVENTS         (unsigned char)  2
#define ACTIVITY_RETRIEVED      (unsigned char)  3
#define CONTROL_RETRIEVED       (unsigned char)  4
#define LIVE_EVENTS             (unsigned char)  5
#define ZERO_DELAY_EVENTS       (unsigned char)  6
#define EVAL_ACTIVATIONS        (unsigned char)  7
#define FDROP_ACTIVATIONS       (unsigned char)  8
#define FDROP_CPU_TIME          (unsigned char)  9
#define PP_ACTIVATIONS          (unsigned char) 10
#define PP_TRAVERSED_MACHINES   (unsigned char) 11
#define CK_FF_EVALUATIONS       (unsigned char) 12
#define FF_ACTIVITY             (unsigned char) 13  
#define FF_EVALUATIONS          (unsigned char) 14  
#define ALLOCATED_DSC           (unsigned char) 15
#define USED_DSC                (unsigned char) 16
#define EVALS                   (unsigned char) 17


/* file types */

#define LOG_FILE (unsigned char)  0
#define HIS_FILE (unsigned char)  1
#define TRC_FILE (unsigned char)  2
#define DSO_FILE (unsigned char)  3


/* file extensions */

#define LOG_EXTENSION  "dlg"
#define HIS_EXTENSION  "dhs"
#define TRC_EXTENSION  "dtr"
#define DSO_EXTENSION  "dso"


/* string allocation sizes */

#define MAX_STR_VALUE_LENGTH    MAX_DATA_SIZE + 5
#define MAX_MESSAGE_LENGTH                    512  /* see shell.h */
#define MAX_NAME_LENGTH                        64
#define MAX_FILE_NAME_LENGTH                   16
#define BUFFER_SIZE                          1024


/* POINTER type definition, used to pass pointers between independent
   functions */

typedef int*  POINTER;


