 

         /*********************************************************/
         /*                                                       */
         /*                 COPYRIGHT (C) 1988 BY                 */
         /* CAD Group of Dipartimento di Automatica e Informatica */
         /*             Politecnico di Torino, Italy.             */
         /*                  ALL RIGHTS RESERVED.                 */
         /*                                                       */
         /* This software is furnished under a license and may be */
         /* used and copied  only in accordance with the terms of */
         /* such license. Unauthorized copies will be prosecuted. */
         /*                                                       */
         /*********************************************************/





/******************************************************************************

    Filename:         loadcode.h

    Author:           Pier Luca Montessoro

    Creation date:    25 January 1988

    Contents:         element definition codes used by the MOZART network
                      compiler and linker


                          *** DOCUMENTATION ***

      This file contains the definitions for the data structures representing
 the network to be simulated used in the MOZART environment.

 IMPORTANT:  All the codes are defined and used as 'unsigned char'. This 
             ensures the correct parameters passing between functions also
             on 16 bits word-width machines.


 ******************************************************************************/




/* extension of network description file */

#define NET_EXTENSION       "dee"


/* data types */

#define MZ_WIRE_VALUE       (unsigned char) 0
#define MZ_INTEGER          (unsigned char) 1
#define MZ_REAL             (unsigned char) 2
#define MZ_CHAR             (unsigned char) 3


/* codes related to the position of the element */

#define MZ_INTERNAL         (unsigned char) 0
#define MZ_INPUT            (unsigned char) 1
#define MZ_OUTPUT           (unsigned char) 2
#define MZ_BIDIRECTIONAL    (unsigned char) 3
#define MZ_PHANTOM          (unsigned char) 4


/* codes related to the kind of carrier or device */

#define MZ_NODE            (unsigned char)  0
#define MZ_WIRE            (unsigned char)  1
#define MZ_REGISTER        (unsigned char)  2
#define MZ_CONST           (unsigned char)  3
#define MZ_TRANSISTOR      (unsigned char)  4
#define MZ_GATE            (unsigned char)  5
#define MZ_MEMORY          (unsigned char)  6
#define MZ_PROCESS         (unsigned char)  7
#define MZ_RT_ASSIGNEMENT  (unsigned char)  8
#define MZ_RT_CONDITION    (unsigned char)  9
#define MZ_RT_CONTROL      (unsigned char) 10
#define MZ_RT_LOAD         (unsigned char) 11
#define MZ_ASSTRIG         (unsigned char) 12
#define MZ_BIT_EXTRACT     (unsigned char) 13
#define MZ_ASSTRIG_CNTRL   (unsigned char) 14

/*added behavioral */
#define MZ_BEHAVIORAL	   (unsigned char) 15


/* transistor codes */

#define MZ_DEPLETION        (unsigned char) 0
#define MZ_N_CHANNEL        (unsigned char) 1
#define MZ_P_CHANNEL        (unsigned char) 2


/* gate codes */

#define MZ_AND             (unsigned char)  0
#define MZ_NAND            (unsigned char)  1
#define MZ_OR              (unsigned char)  2
#define MZ_NOR             (unsigned char)  3
#define MZ_BUF             (unsigned char)  4
#define MZ_INV             (unsigned char)  5
#define MZ_AND_TRI         (unsigned char)  6
#define MZ_NAND_TRI        (unsigned char)  7
#define MZ_OR_TRI          (unsigned char)  8
#define MZ_NOR_TRI         (unsigned char)  9
#define MZ_BUF_TRI         (unsigned char) 10
#define MZ_INV_TRI         (unsigned char) 11
#define MZ_AND_TRIN        (unsigned char) 12
#define MZ_NAND_TRIN       (unsigned char) 13
#define MZ_OR_TRIN         (unsigned char) 14
#define MZ_NOR_TRIN        (unsigned char) 15
#define MZ_BUF_TRIN        (unsigned char) 16
#define MZ_INV_TRIN        (unsigned char) 17
#define MZ_AND_OC          (unsigned char) 18
#define MZ_NAND_OC         (unsigned char) 19
#define MZ_OR_OC           (unsigned char) 20
#define MZ_NOR_OC          (unsigned char) 21
#define MZ_BUF_OC          (unsigned char) 22
#define MZ_INV_OC          (unsigned char) 23
#define MZ_AND_OE          (unsigned char) 24
#define MZ_NAND_OE         (unsigned char) 25
#define MZ_OR_OE           (unsigned char) 26
#define MZ_NOR_OE          (unsigned char) 27
#define MZ_BUF_OE          (unsigned char) 28
#define MZ_INV_OE          (unsigned char) 29
#define MZ_XOR             (unsigned char) 30
#define MZ_XNOR            (unsigned char) 31
#define MZ_MUX             (unsigned char) 32
#define MZ_SRNOR           (unsigned char) 33
#define MZ_SRNAND          (unsigned char) 34
#define MZ_TGP             (unsigned char) 35
#define MZ_TGN             (unsigned char) 36
#define MZ_MTGP            (unsigned char) 37
#define MZ_MTGN            (unsigned char) 38
#define MZ_BUS_TRI         (unsigned char) 39
#define MZ_BUS_MOS         (unsigned char) 40
#define MZ_BUS_UP          (unsigned char) 41
#define MZ_BUS_DOWN        (unsigned char) 42


/*user behavioral codes */

#define MZ_HALF_ADDER	   (unsigned char) 1
#define MZ_FULL_ADDER	   (unsigned char) 2
#define MZ_RING_BUFFER     (unsigned char) 3
#define MZ_4TO1_MUX	   (unsigned char) 4
#define MZ_S27        	   (unsigned char) 5


/* flip-flop codes */

#define MZ_JKBLOCK         (unsigned char) 43
#define MZ_DFF_PET         (unsigned char) 44
#define MZ_TFF_PET         (unsigned char) 45
#define MZ_JKFF_PET        (unsigned char) 46
#define MZ_DFF_PET_C       (unsigned char) 47
#define MZ_TFF_PET_C       (unsigned char) 48
#define MZ_JKFF_PET_C      (unsigned char) 49
#define MZ_DFF_PET_S       (unsigned char) 50
#define MZ_TFF_PET_S       (unsigned char) 51 
#define MZ_JKFF_PET_S      (unsigned char) 52 
#define MZ_DFF_PET_SC      (unsigned char) 53
#define MZ_TFF_PET_SC      (unsigned char) 54
#define MZ_JKFF_PET_SC     (unsigned char) 55
#define MZ_DFF_NET         (unsigned char) 56
#define MZ_TFF_NET         (unsigned char) 57
#define MZ_JKFF_NET        (unsigned char) 58
#define MZ_DFF_NET_C       (unsigned char) 59
#define MZ_TFF_NET_C       (unsigned char) 60
#define MZ_JKFF_NET_C      (unsigned char) 61
#define MZ_DFF_NET_S       (unsigned char) 62
#define MZ_TFF_NET_S       (unsigned char) 63
#define MZ_JKFF_NET_S      (unsigned char) 64
#define MZ_DFF_NET_SC      (unsigned char) 65
#define MZ_TFF_NET_SC      (unsigned char) 66
#define MZ_JKFF_NET_SC     (unsigned char) 67
#define MZ_LATCH_HIGH      (unsigned char) 68
#define MZ_LATCH_LOW       (unsigned char) 69


/* memory codes */

#define MZ_PLA             (unsigned char)  0
#define MZ_RAM             (unsigned char)  1
#define MZ_ROM             (unsigned char)  2


/* computational appendix */

#define MZ_STOP            (unsigned char)  0
#define MZ_OPERATOR        (unsigned char)  1
#define MZ_OPERAND         (unsigned char)  2
#define MZ_CONSTANT        (unsigned char)  3
#define MZ_EFUNCTION       (unsigned char)  4


/* stop codes */

#define MZ_ASSIGN          (unsigned char)  0
#define MZ_ENABLE          (unsigned char)  1


/* operator codes */

#define MZ_ADD             (unsigned char)  0
#define MZ_ADC             (unsigned char)  1
#define MZ_SUB             (unsigned char)  2
#define MZ_SBB             (unsigned char)  3
#define MZ_MPY             (unsigned char)  4
#define MZ_DIV             (unsigned char)  5
#define MZ_ROR             (unsigned char)  6
#define MZ_ROL             (unsigned char)  7
#define MZ_SHL             (unsigned char)  8
#define MZ_SRA             (unsigned char)  9
#define MZ_SRL             (unsigned char) 10
#define MZ_DEC             (unsigned char) 12
#define MZ_FAN             (unsigned char) 13
#define MZ_BIT_AND         (unsigned char) 14
#define MZ_BIT_OR          (unsigned char) 15
#define MZ_BIT_NOT         (unsigned char) 16
#define MZ_BIT_XOR         (unsigned char) 17
#define MZ_COMPR_AND       (unsigned char) 18
#define MZ_COMPR_OR        (unsigned char) 19
#define MZ_COMPR_XOR       (unsigned char) 20
#define MZ_CHS             (unsigned char) 21
#define MZ_IS_ZERO         (unsigned char) 22
#define MZ_IS_ONE          (unsigned char) 23
#define MZ_IS_DRIVEN       (unsigned char) 24
#define MZ_TO_CAR          (unsigned char) 25
#define MZ_CONCAT          (unsigned char) 26
#define MZ_EXTRACT         (unsigned char) 27
#define MZ_EQ              (unsigned char) 28
#define MZ_NE              (unsigned char) 29
#define MZ_GT              (unsigned char) 30
#define MZ_GE              (unsigned char) 31
#define MZ_LT              (unsigned char) 32
#define MZ_LE              (unsigned char) 33
#define MZ_AND_FUNCTION    (unsigned char) 34
#define MZ_OR_FUNCTION     (unsigned char) 35
#define MZ_NOT_FUNCTION    (unsigned char) 36
#define MZ_XOR_FUNCTION    (unsigned char) 37
#define MZ_TRIGGER_0       (unsigned char) 38
#define MZ_TRIGGER_1       (unsigned char) 39


/* fanin type codes */

#define MZ_FI_EN_0          (unsigned char) 0
#define MZ_FI_EN_1          (unsigned char) 1
#define MZ_FI_TR_0          (unsigned char) 2
#define MZ_FI_TR_1          (unsigned char) 3
#define MZ_FI_DATA          (unsigned char) 4


